@@ -40,7 +40,7 @@ Pwnlib Developers
 -----------------
 A module-specific logger can be imported into the module via::
 
-    from .log import getLogger
+    from pwnlib.log import getLogger
     log = getLogger(__name__)
 
 This provides an easy way to filter logging programmatically
@@ -91,10 +91,7 @@ unless :data:`context.log_level` is also set to ``'DEBUG'``.  Other handlers
 will however see the extra log records generated by the ``'pwnlib.tubes.ssh'``
 logger.
 """
-
-__all__ = [
-    'getLogger', 'install_default_handler', 'rootlogger'
-]
+from __future__ import absolute_import
 
 import ConfigParser
 import logging
@@ -105,15 +102,22 @@ import sys
 import threading
 import time
 
-from . import term
-from .context import Thread
-from .context import context
-from .exception import PwnlibException
-from .term import spinners
-from .term import text
+from pwnlib import term
+from pwnlib.config import register_config
+from pwnlib.context import Thread
+from pwnlib.context import context
+from pwnlib.exception import PwnlibException
+from pwnlib.term import spinners
+from pwnlib.term import text
+
+__all__ = [
+    'getLogger', 'install_default_handler', 'rootlogger'
+]
+
+
 
 # list of prefixes to use for the different message types.  note that the `text`
-# module won't add any escape codes if `sys.stderr.isatty()` is `False`
+# module won't add any escape codes if `pwnlib.context.log_console.isatty()` is `False`
 _msgtype_prefixes = {
     'status'       : [text.magenta, 'x'],
     'success'      : [text.bold_green, '+'],
@@ -129,26 +133,26 @@ _msgtype_prefixes = {
     }
 
 
-# permit setting logging colors from a configuration file
-config = os.path.expanduser('~/.pwn.conf')
-if os.path.exists(config):
-    c = ConfigParser.ConfigParser()
-    c.read([config])
-
-    for section in c.sections():
-        if section not in _msgtype_prefixes:
+def read_log_config(settings):
+    log = getLogger(__name__)
+    for key, value in settings.items():
+        if '.' not in key:
+            log.warn("Invalid configuration option %r in section %r" % (key, 'log'))
             continue
 
-        for key, value in c.items(section):
-            if key == 'color':
-                try:
-                    _msgtype_prefixes[section][0] = getattr(text, value)
-                except AttributeError:
-                    pass
+        msgtype, key = key.split('.', 1)
 
-            elif key == 'symbol':
-                _msgtype_prefixes[section][1] = value
+        if key == 'color':
+            current = _msgtype_prefixes[msgtype][0]
+            _msgtype_prefixes[msgtype][0] = getattr(text, value, current)
 
+        elif key == 'symbol':
+            _msgtype_prefixes[msgtype][1] = value
+
+        else:
+            log.warn("Unknown configuration option %r in section %r" % (key, 'log'))
+
+register_config('log', read_log_config)
 
 # the text decoration to use for spinners.  the spinners themselves can be found
 # in the `pwnlib.term.spinners` module
@@ -166,7 +170,6 @@ class Progress(object):
     using :meth:`Logger.progress`.
     """
     def __init__(self, logger, msg, status, level, args, kwargs):
-        global _progressid
         self._logger = logger
         self._msg = msg
         self._status = status
@@ -264,7 +267,7 @@ class Logger(object):
             # This is a minor hack to permit user-defined classes which inherit
             # from a tube (which do not actually reside in the pwnlib library)
             # to receive logging abilities that behave as they would expect from
-            # the resto f the library
+            # the rest of the library
             module = self.__module__
             if not module.startswith('pwnlib'):
                 module = 'pwnlib.' + module
@@ -490,10 +493,14 @@ class Handler(logging.StreamHandler):
     logger will not be emitted but rather an animated progress line will be
     created.
 
-    This handler outputs to ``sys.stderr``.
-
     An instance of this handler is added to the ``'pwnlib'`` logger.
     """
+    @property
+    def stream(self):
+        return context.log_console
+    @stream.setter
+    def stream(self, value):
+        pass
     def emit(self, record):
         """
         Emit a log record or create/update an animated progress logger
@@ -652,7 +659,7 @@ log_file.setFormatter(logging.Formatter(fmt, iso_8601))
 #     logger.addHandler(myCoolPitchingHandler)
 #
 rootlogger = getLogger('pwnlib')
-console   = Handler(sys.stdout)
+console   = Handler()
 formatter = Formatter()
 console.setFormatter(formatter)
 
@@ -663,8 +670,6 @@ def install_default_handler():
     the ``pwnlib`` root logger.  This function is automatically called from when
     importing :mod:`pwn`.
     '''
-    console.stream = sys.stdout
-
     logger         = logging.getLogger('pwnlib')
 
     if console not in logger.handlers:
