@@ -9,13 +9,14 @@ in :class:`pwnlib.tubes.tube`. The remaining classes should only implement
 just enough for the class to work and possibly code pertaining only to
 that specific kind of tube.
 """
+from __future__ import absolute_import
+
+from pwnlib.tubes import listen
+from pwnlib.tubes import process
+from pwnlib.tubes import remote
+from pwnlib.tubes import serialtube
+from pwnlib.tubes import sock
+from pwnlib.tubes import ssh
+from pwnlib.tubes import tube
 
 __all__ = ['tube', 'sock', 'remote', 'listen', 'process', 'serialtube', 'ssh']
-
-from . import listen
-from . import process
-from . import remote
-from . import serialtube
-from . import sock
-from . import ssh
-from . import tube
