@@ -1,6 +1,18 @@
 """
 This module includes and extends the standard module :mod:`itertools`.
 """
+from __future__ import absolute_import
+
+import collections
+import copy
+import multiprocessing
+import operator
+import random
+import time
+from itertools import *
+
+from pwnlib.context import context
+from pwnlib.log import getLogger
 
 __all__ = [
     'bruteforce'                             ,
@@ -53,16 +65,7 @@ __all__ = [
     'tee'
 ]
 
-import collections
-import copy
-import multiprocessing
-import operator
-import random
-import time
-from itertools import *
 
-from ..context import context
-from ..log import getLogger
 
 log = getLogger(__name__)
 
@@ -178,11 +181,11 @@ def quantify(iterable, pred = bool):
     Arguments:
         iterable:  An iterable.
         pred:  A function that given an element from `iterable` returns either
-               ``True`` or ``False``.
+               :const:`True` or :const:`False`.
 
     Returns:
       The number of elements in `iterable` for which `pred` returns
-      ``True``.
+      :const:`True`.
 
     Examples:
       >>> quantify([1, 2, 3, 4], lambda x: x % 2 == 0)
