@@ -7,10 +7,10 @@ import socket
 import stat
 import string
 
-from . import lists
-from . import fiddling
-from ..context import context
-from ..log import getLogger
+from pwnlib.context import context
+from pwnlib.log import getLogger
+from pwnlib.util import fiddling
+from pwnlib.util import lists
 
 log = getLogger(__name__)
 
@@ -51,14 +51,15 @@ def binary_ip(host):
     return socket.inet_aton(socket.gethostbyname(host))
 
 
-def size(n, abbriv = 'B', si = False):
-    """size(n, abbriv = 'B', si = False) -> str
+def size(n, abbrev = 'B', si = False):
+    """size(n, abbrev = 'B', si = False) -> str
 
     Convert the length of a bytestream to human readable form.
 
     Arguments:
-      n(int,str): The length to convert to human readable form
-      abbriv(str):
+      n(int,iterable): The length to convert to human readable form,
+        or an object which can have ``len()`` called on it.
+      abbrev(str): String appended to the size, defaults to ``'B'``.
 
     Example:
         >>> size(451)
@@ -67,24 +68,30 @@ def size(n, abbriv = 'B', si = False):
         '1000B'
         >>> size(1024)
         '1.00KB'
+        >>> size(1024, ' bytes')
+        '1.00K bytes'
         >>> size(1024, si = True)
         '1.02KB'
         >>> [size(1024 ** n) for n in range(7)]
         ['1B', '1.00KB', '1.00MB', '1.00GB', '1.00TB', '1.00PB', '1024.00PB']
+        >>> size([])
+        '0B'
+        >>> size([1,2,3])
+        '3B'
     """
-    if isinstance(n, str):
+    if hasattr(n, '__len__'):
         n = len(n)
 
     base = 1000.0 if si else 1024.0
     if n < base:
-        return '%d%s' % (n, abbriv)
+        return '%d%s' % (n, abbrev)
 
     for suffix in ['K', 'M', 'G', 'T']:
         n /= base
         if n < base:
-            return '%.02f%s%s' % (n, suffix, abbriv)
+            return '%.02f%s%s' % (n, suffix, abbrev)
 
-    return '%.02fP%s' % (n / base, abbriv)
+    return '%.02fP%s' % (n / base, abbrev)
 
 KB = 1024
 MB = 1024 * KB
@@ -174,61 +181,80 @@ def run_in_new_terminal(command, terminal = None, args = None):
 
     Run a command in a new terminal.
 
-    When `terminal` is not set:
-      - If `context.terminal` is set it will be used.  If it is an iterable then
-        `context.terminal[1:]` are default arguments.
-      - If X11 is detected (by the presence of the ``DISPLAY`` environment
-        variable), ``x-terminal-emulator`` is used.
-      - If tmux is detected (by the presence of the ``TMUX`` environment
-        variable), a new pane will be opened.
+    When ``terminal`` is not set:
+        - If ``context.terminal`` is set it will be used.
+          If it is an iterable then ``context.terminal[1:]`` are default arguments.
+        - If a ``pwntools-terminal`` command exists in ``$PATH``, it is used
+        - If ``$TERM_PROGRAM`` is set, that is used.
+        - If X11 is detected (by the presence of the ``$DISPLAY`` environment
+          variable), ``x-terminal-emulator`` is used.
+        - If tmux is detected (by the presence of the ``$TMUX`` environment
+          variable), a new pane will be opened.
 
     Arguments:
-      command (str): The command to run.
-      terminal (str): Which terminal to use.
-      args (list): Arguments to pass to the terminal
+        command (str): The command to run.
+        terminal (str): Which terminal to use.
+        args (list): Arguments to pass to the terminal
+
+    Note:
+        The command is opened with ``/dev/null`` for stdin, stdout, stderr.
 
     Returns:
-      None
-
+      PID of the new terminal process
     """
 
     if not terminal:
         if context.terminal:
             terminal = context.terminal[0]
             args     = context.terminal[1:]
-        elif 'DISPLAY' in os.environ:
+        elif which('pwntools-terminal'):
+            terminal = 'pwntools-terminal'
+            args     = []
+        elif 'TERM_PROGRAM' in os.environ:
+            terminal = os.environ['TERM_PROGRAM']
+            args     = []
+        elif 'DISPLAY' in os.environ and which('x-terminal-emulator'):
             terminal = 'x-terminal-emulator'
             args     = ['-e']
-        elif 'TMUX' in os.environ:
+        elif 'TMUX' in os.environ and which('tmux'):
             terminal = 'tmux'
             args     = ['splitw']
 
     if not terminal:
-        log.error('Argument `terminal` is not set, and could not determine a default')
+        log.error('Could not find a terminal binary to use. Set context.terminal to your terminal.')
+    elif not which(terminal):
+        log.error('Could not find terminal binary %r. Set context.terminal to your terminal.' % terminal)
 
-    terminal_path = which(terminal)
+    if isinstance(args, tuple):
+        args = list(args)
 
-    if not terminal_path:
-        log.error('Could not find terminal: %s' % terminal)
-
-    argv = [terminal_path] + args
+    argv = [which(terminal)] + args
 
     if isinstance(command, str):
+        if ';' in command:
+            log.error("Cannot use commands with semicolon.  Create a script and invoke that directly.")
         argv += [command]
     elif isinstance(command, (list, tuple)):
+        if any(';' in c for c in command):
+            log.error("Cannot use commands with semicolon.  Create a script and invoke that directly.")
         argv += list(command)
 
     log.debug("Launching a new terminal: %r" % argv)
 
-    if os.fork() == 0:
+    pid = os.fork()
+
+    if pid == 0:
         # Closing the file descriptors makes everything fail under tmux on OSX.
         if platform.system() != 'Darwin':
-            os.close(0)
-            os.close(1)
-            os.close(2)
+            devnull = open(os.devnull, 'rwb')
+            os.dup2(devnull.fileno(), 0)
+            os.dup2(devnull.fileno(), 1)
+            os.dup2(devnull.fileno(), 2)
         os.execv(argv[0], argv)
         os._exit(1)
 
+    return pid
+
 def parse_ldd_output(output):
     """Parses the output from a run of 'ldd' on a binary.
     Returns a dictionary of {path: address} for
@@ -271,163 +297,6 @@ def mkdir_p(path):
         else:
             raise
 
-def sh_string(s):
-    """Outputs a string in a format that will be understood by /bin/sh.
-
-    If the string does not contain any bad characters, it will simply be
-    returned, possibly with quotes. If it contains bad characters, it will
-    be escaped in a way which is compatible with most known systems.
-
-    Examples:
-
-        >>> print sh_string('foobar')
-        foobar
-        >>> print sh_string('foo bar')
-        'foo bar'
-        >>> print sh_string("foo'bar")
-        "foo'bar"
-        >>> print sh_string("foo\\\\bar")
-        'foo\\bar'
-        >>> print sh_string("foo\\\\'bar")
-        "foo\\\\'bar"
-        >>> print sh_string("foo\\x01'bar")
-        "$(printf 'foo\\001\\047bar')"
-        >>> print `subprocess.check_output("echo -n " + sh_string("foo\\x01'bar"), shell = True)`
-        "foo\\x01'bar"
-    """
-
-    very_good = set(string.ascii_letters + string.digits)
-    good      = (very_good | set(string.punctuation + ' ')) - set("'")
-    alt_good  = (very_good | set(string.punctuation + ' ')) - set('!')
-
-    if '\x00' in s:
-        log.error("sh_string(): Cannot create a null-byte")
-    if s.endswith('\n'):
-        log.error("sh_string(): Cannot create a newline-terminated string")
-
-    if all(c in very_good for c in s):
-        return s
-    elif all(c in good for c in s):
-        return "'%s'" % s
-    elif all(c in alt_good for c in s):
-        fixed = ''
-        for c in s:
-            if c in '"\\$`':
-                fixed += '\\' + c
-            else:
-                fixed += c
-        return '"%s"' % fixed
-    else:
-        fixed = ''
-        for c in s:
-            if c == '\\':
-                fixed += '\\\\'
-            elif c == '\n':
-                fixed += '\\n'
-            elif c in good:
-                fixed += c
-            else:
-                fixed += '\\%03o' % ord(c)
-        return '"$(printf \'%s\')"' % fixed
-
-def sh_prepare(variables, export = False):
-    """Outputs a posix compliant shell command that will put the data specified
-    by the dictionary into the environment.
-
-    It is assumed that the keys in the dictionary are valid variable names that
-    does not need any escaping.
-
-    Arguments:
-      variables(dict): The variables to set.
-      export(bool): Should the variables be exported or only stored in the shell environment?
-      output(str): A valid posix shell command that will set the given variables.
-
-    It is assumed that `var` is a valid name for a variable in the shell.
-
-    Examples:
-
-        >>> print sh_prepare({'X': 'foobar'})
-        X=foobar
-        >>> r = sh_prepare({'X': 'foobar', 'Y': 'cookies'})
-        >>> r == 'X=foobar;Y=cookies' or r == 'Y=cookies;X=foobar'
-        True
-        >>> print sh_prepare({'X': 'foo bar'})
-        X='foo bar'
-        >>> print sh_prepare({'X': "foo'bar"})
-        X="foo'bar"
-        >>> print sh_prepare({'X': "foo\\\\bar"})
-        X='foo\\bar'
-        >>> print sh_prepare({'X': "foo\\\\'bar"})
-        X="foo\\\\'bar"
-        >>> print sh_prepare({'X': "foo\\x01'bar"})
-        X="$(printf 'foo\\001\\047bar')"
-        >>> print sh_prepare({'X': "foo\\x01'bar"}, export = True)
-        export X="$(printf 'foo\\001\\047bar')"
-        >>> print sh_prepare({'X': "foo\\x01'bar\\n"})
-        X="$(printf 'foo\\001\\047bar\\nx')";X=${X%x}
-        >>> print sh_prepare({'X': "foo\\x01'bar\\n"}, export = True)
-        X="$(printf 'foo\\001\\047bar\\nx')";export X=${X%x}
-        >>> print `subprocess.check_output('%s;echo -n "$X"' % sh_prepare({'X': "foo\\x01'bar"}), shell = True)`
-        "foo\\x01'bar"
-    """
-
-    out = []
-    export = 'export ' if export else ''
-
-    for k, v in variables.items():
-        if v.endswith('\n'):
-            out.append('%s=%s;%s%s=${%s%%x}' % (k, sh_string(v + "x"), export, k, k))
-        else:
-            out.append('%s%s=%s' % (export, k, sh_string(v)))
-    return ';'.join(out)
-
-def sh_command_with(f, *args):
-    """sh_command_with(f, arg0, ..., argN) -> command
-
-    Returns a command create by evaluating `f(new_arg0, ..., new_argN)`
-    whenever `f` is a function and `f % (new_arg0, ..., new_argN)` otherwise.
-
-    If the arguments are purely alphanumeric, then they are simply passed to
-    function. If they are simple to escape, they will be escaped and passed to
-    the function.
-
-    If the arguments contain trailing newlines, then it is hard to use them
-    directly because of a limitation in the posix shell. In this case the
-    output from `f` is prepended with a bit of code to create the variables.
-
-    Examples:
-
-        >>> print sh_command_with(lambda: "echo hello")
-        echo hello
-        >>> print sh_command_with(lambda x: "echo " + x, "hello")
-        echo hello
-        >>> print sh_command_with(lambda x: "echo " + x, "\\x01")
-        echo "$(printf '\\001')"
-        >>> import random
-        >>> random.seed(1)
-        >>> print sh_command_with(lambda x: "echo " + x, "\\x01\\n")
-        dwtgmlqu="$(printf '\\001\\nx')";dwtgmlqu=${dwtgmlqu%x};echo "$dwtgmlqu"
-        >>> random.seed(1)
-        >>> print sh_command_with("echo %s", "\\x01\\n")
-        dwtgmlqu="$(printf '\\001\\nx')";dwtgmlqu=${dwtgmlqu%x};echo "$dwtgmlqu"
-    """
-
-    args = list(args)
-    out = []
-
-    for n in range(len(args)):
-        if args[n].endswith('\n'):
-            v = fiddling.randoms(8)
-            out.append(sh_prepare({v: args[n]}))
-            args[n] = '"$%s"' % v
-        else:
-            args[n] = sh_string(args[n])
-    if hasattr(f, '__call__'):
-        out.append(f(*args))
-    else:
-        out.append(f % tuple(args))
-    return ';'.join(out)
-
 def dealarm_shell(tube):
     """Given a tube which is a shell, dealarm it.
     """
